@@ -1,5 +1,4 @@
-/* Time-stamp: <2006-06-28 01:25:27 jcs>
-|
+/*
 |  Copyright (C) 2002-2005 Jorg Schuler <jcsjcs at users sourceforge net>
 |  Part of the gtkpod project.
 | 
@@ -24,7 +23,7 @@
 | 
 |  This product is not supported/written/published by Apple!
 |
-|  $Id: display_playlists.c,v 1.92 2006/06/27 16:29:43 jcsjcs Exp $
+|  $Id: display_playlists.c 1596 2007-06-26 14:33:01Z dforsi $
 */
 
 #ifdef HAVE_CONFIG_H
@@ -66,7 +65,12 @@ static GtkTargetEntry pm_drop_types [] = {
 };
 
 
-void pm_rows_reordered (void);
+static void pm_rows_reordered (void);
+static GtkTreePath *pm_get_path_for_itdb (iTunesDB *itdb);
+static GtkTreePath *pm_get_path_for_playlist (Playlist *pl);
+static gint pm_get_position_for_playlist (Playlist *pl);
+static gboolean pm_get_iter_for_itdb (iTunesDB *itdb, GtkTreeIter *iter);
+static gboolean pm_get_iter_for_playlist (Playlist *pl, GtkTreeIter *iter);
 
 
 
@@ -182,6 +186,8 @@ static gboolean pm_drag_motion (GtkWidget *widget,
     GdkAtom target;
     guint info;
     Playlist *pl_d;
+    iTunesDB *itdb;
+    ExtraiTunesDBData *eitdb;
 
     g_return_val_if_fail (widget, FALSE);
     g_return_val_if_fail (GTK_IS_TREE_VIEW (widget), FALSE);
@@ -206,17 +212,29 @@ static gboolean pm_drag_motion (GtkWidget *widget,
 
     if(gtk_tree_model_get_iter (model, &iter_d, path))
     {
-	gtk_tree_model_get (model, &iter_d, 0, &pl_d, -1);
+	gtk_tree_model_get (model, &iter_d, PM_COLUMN_PLAYLIST, &pl_d, -1);
     }
     g_return_val_if_fail (pl_d, FALSE);
+    itdb = pl_d->itdb;
+    g_return_val_if_fail (itdb, FALSE);
+    eitdb = itdb->userdata;
+    g_return_val_if_fail (eitdb, FALSE);
 
     target = gtk_drag_dest_find_target (widget, dc, NULL);
 
+    /* no drop possible if repository is not loaded */
+    if (!eitdb->itdb_imported)
+    {
+	gtk_tree_path_free (path);
+	gdk_drag_status (dc, 0, time);
+	return FALSE;
+    }
+
     /* no drop possible if no valid target can be found */
     if (target == GDK_NONE)
     {
-	gdk_drag_status (dc, 0, time);
 	gtk_tree_path_free (path);
+	gdk_drag_status (dc, 0, time);
 	return FALSE;
     }
 
@@ -225,8 +243,8 @@ static gboolean pm_drag_motion (GtkWidget *widget,
     {   /* MPL */
 	if (pos == GTK_TREE_VIEW_DROP_BEFORE)
 	{
-	    gdk_drag_status (dc, 0, time);
 	    gtk_tree_path_free (path);
+	    gdk_drag_status (dc, 0, time);
 	    return FALSE;
 	}
     }
@@ -235,8 +253,8 @@ static gboolean pm_drag_motion (GtkWidget *widget,
     if (!gtk_target_list_find (
 	    gtk_drag_dest_get_target_list (widget), target, &info))
     {
-	gdk_drag_status (dc, 0, time);
 	gtk_tree_path_free (path);
+	gdk_drag_status (dc, 0, time);
 	return FALSE;
     }
 
@@ -255,8 +273,8 @@ static gboolean pm_drag_motion (GtkWidget *widget,
 	    if ((pos == GTK_TREE_VIEW_DROP_INTO_OR_BEFORE) ||
 		(pos == GTK_TREE_VIEW_DROP_INTO_OR_AFTER))
 	    {
-		gdk_drag_status (dc, 0, time);
 		gtk_tree_path_free (path);
+		gdk_drag_status (dc, 0, time);
 		return FALSE;
 	    }
 	}
@@ -295,7 +313,7 @@ on_pm_dnd_get_file_foreach(GtkTreeModel *tm, GtkTreePath *tp,
     g_return_if_fail (data);
 
     /* get current playlist */
-    gtk_tree_model_get(tm, iter, PM_COLUMN_PLAYLIST, &pl, -1); 
+    gtk_tree_model_get (tm, iter, PM_COLUMN_PLAYLIST, &pl, -1); 
     g_return_if_fail (pl);
 
     for (gl=pl->members; gl; gl=gl->next)
@@ -330,7 +348,7 @@ on_pm_dnd_get_uri_foreach(GtkTreeModel *tm, GtkTreePath *tp,
     g_return_if_fail (data);
 
     /* get current playlist */
-    gtk_tree_model_get(tm, iter, PM_COLUMN_PLAYLIST, &pl, -1); 
+    gtk_tree_model_get (tm, iter, PM_COLUMN_PLAYLIST, &pl, -1); 
     g_return_if_fail (pl);
 
     for (gl=pl->members; gl; gl=gl->next)
@@ -578,7 +596,7 @@ static void pm_drag_data_received (GtkWidget       *widget,
 				   gpointer         user_data)
 {
     GtkTreeIter iter_d, iter_s;
-    GtkTreePath *path_d=NULL, *path_s;
+    GtkTreePath *path_d=NULL;
     GtkTreePath *path_m;
     GtkTreeModel *model;
     GtkTreeViewDropPosition pos = 0;
@@ -614,7 +632,7 @@ static void pm_drag_data_received (GtkWidget       *widget,
 	g_object_set_data (G_OBJECT (widget), "drag_data_by_motion_pos", NULL);
 	if(gtk_tree_model_get_iter (model, &iter_d, path_m))
 	{
-	    gtk_tree_model_get (model, &iter_d, 0, &pl, -1);
+	    gtk_tree_model_get (model, &iter_d, PM_COLUMN_PLAYLIST, &pl, -1);
 	}
 	gtk_tree_path_free (path_m);
 
@@ -673,27 +691,17 @@ static void pm_drag_data_received (GtkWidget       *widget,
     }
     g_return_if_fail (path_d);
 
-    if(gtk_tree_model_get_iter(model, &iter_d, path_d))
+    if(gtk_tree_model_get_iter (model, &iter_d, path_d))
     {
-	gtk_tree_model_get(model, &iter_d, 0, &pl, -1);
-    }
-    g_return_if_fail (pl);
-
-    /* get position of current path */
-    if (gtk_tree_path_get_depth (path_d) == 1)
-    {   /* MPL */
-	position = 0;
-    }
-    else
-    {
-	gint *indices = gtk_tree_path_get_indices (path_d);
-	/* need to add 1 because MPL is one level higher and not
-	   counted */
-	position = indices[1] + 1;
+	gtk_tree_model_get (model, &iter_d, PM_COLUMN_PLAYLIST, &pl, -1);
     }
     gtk_tree_path_free (path_d);
     path_d = NULL;
 
+    g_return_if_fail (pl);
+
+    position = pm_get_position_for_playlist (pl);
+
 /*  printf("position: %d\n", position); */
     switch (info)
     {
@@ -846,26 +854,28 @@ static void pm_drag_data_received (GtkWidget       *widget,
 		gtk_drag_finish (dc, TRUE, FALSE, time);
 		break;
 	    case GDK_ACTION_MOVE:
-		if (prefs_get_int("pm_sort") != SORT_NONE)
-		{
-		    gtkpod_statusbar_message (_("Can't reorder sorted treeview."));
-		    gtk_drag_finish (dc, FALSE, FALSE, time);
-		    return;
-		}
-		path_s = pm_get_path (pl_s);
-		g_return_if_fail (path_s);
-		g_return_if_fail (gtk_tree_model_get_iter (
-				      model, &iter_s, path_s));
-		gtk_tree_path_free (path_s);
+		pm_get_iter_for_playlist (pl_s, &iter_s);
 		switch (pos)
 		{
 		case GTK_TREE_VIEW_DROP_BEFORE:
+		    if (prefs_get_int("pm_sort") != SORT_NONE)
+		    {
+			gtkpod_statusbar_message (_("Can't reorder sorted treeview."));
+			gtk_drag_finish (dc, FALSE, FALSE, time);
+			return;
+		    }
 		    gtk_tree_store_move_before (GTK_TREE_STORE (model),
 						&iter_s, &iter_d);
 		    pm_rows_reordered ();
 		    gtk_drag_finish (dc, TRUE, FALSE, time);
 		    break;
 		case GTK_TREE_VIEW_DROP_AFTER:
+		    if (prefs_get_int("pm_sort") != SORT_NONE)
+		    {
+			gtkpod_statusbar_message (_("Can't reorder sorted treeview."));
+			gtk_drag_finish (dc, FALSE, FALSE, time);
+			return;
+		    }
 		    gtk_tree_store_move_after (GTK_TREE_STORE (model),
 					       &iter_s, &iter_d);
 		    pm_rows_reordered ();
@@ -875,7 +885,7 @@ static void pm_drag_data_received (GtkWidget       *widget,
 		    pl_d = pl;
 		    if (pl_d != pl_s)
 			add_trackglist_to_playlist (pl_d, pl_s->members);
-		    gtk_drag_finish (dc, TRUE, TRUE, time);
+		    gtk_drag_finish (dc, TRUE, FALSE, time);
 		    break;
 		}
 		break;
@@ -883,8 +893,6 @@ static void pm_drag_data_received (GtkWidget       *widget,
 		gtk_drag_finish (dc, FALSE, FALSE, time);
 		g_return_if_reached ();
 	    }
-	    pm_rows_reordered ();
-	    return;
 	}
 	else
 	{   /*handle DND between two itdbs */
@@ -945,6 +953,9 @@ static void pm_drag_data_received (GtkWidget       *widget,
 	gtk_drag_finish (dc, FALSE, FALSE, time);
 	break;
     }
+
+    /* display if any duplicates were skipped */
+    gp_duplicate_remove (NULL, NULL);
 }
 
 
@@ -991,6 +1002,105 @@ on_playlist_treeview_key_release_event (GtkWidget       *widget,
 }
 
 
+/* ---------------------------------------------------------------- */
+/* Section for playlist display helper functions                    */
+/* ---------------------------------------------------------------- */
+
+
+/* Find the iter that represents the repository @itdb
+ *
+ * Return TRUE if the repository could be found. In that case @itdb_iter
+ * will be set to the corresponding iter. The value of @itdb_iter is
+ * undefined when the repository couldn't be found, in which case FALSE
+ * is returned. */
+static gboolean pm_get_iter_for_itdb (iTunesDB *itdb, GtkTreeIter *itdb_iter)
+{
+    GtkTreeModel *model;
+
+    g_return_val_if_fail (playlist_treeview, FALSE);
+    g_return_val_if_fail (itdb, FALSE);
+    g_return_val_if_fail (itdb_iter, FALSE);
+
+    model = GTK_TREE_MODEL (gtk_tree_view_get_model (playlist_treeview));
+
+    if (gtk_tree_model_get_iter_first (model, itdb_iter))
+    {
+	do
+	{
+	    iTunesDB *itdb_model;
+	    gtk_tree_model_get (model, itdb_iter,
+				PM_COLUMN_ITDB, &itdb_model,
+				-1);
+	    g_return_val_if_fail (itdb_model, FALSE);
+	    if (itdb == itdb_model)
+	    {
+		return TRUE;
+	    }
+	} while (gtk_tree_model_iter_next (model, itdb_iter));
+    }
+    return FALSE;
+}
+
+
+/* Find the iter that contains Playlist @playlist
+ *
+ * Return TRUE if the playlist could be found. In that case @pl_iter
+ * will be set to the corresponding iter. The value of @pl_iter is
+ * undefined when the playlist couldn't be found, in which case FALSE
+ * is returned. */
+static gboolean pm_get_iter_for_playlist (Playlist *playlist, GtkTreeIter *pl_iter)
+{
+    GtkTreeIter itdb_iter;
+
+    g_return_val_if_fail (playlist_treeview, FALSE);
+    g_return_val_if_fail (playlist, FALSE);
+    g_return_val_if_fail (pl_iter, FALSE);
+
+    /* First get the iter with the itdb in it */
+
+    if (pm_get_iter_for_itdb (playlist->itdb, &itdb_iter))
+    {
+	GtkTreeModel *model;
+	Playlist *pl;
+
+	model = GTK_TREE_MODEL (gtk_tree_view_get_model (playlist_treeview));
+
+	/* Check if this is already the right iter */
+	gtk_tree_model_get (model, &itdb_iter,
+			    PM_COLUMN_PLAYLIST, &pl,
+			    -1);
+	g_return_val_if_fail (pl, FALSE);
+	if (pl == playlist)
+	{
+	    *pl_iter = itdb_iter;
+	    return TRUE;
+	}
+
+	/* no -- go down one hierarchy and try all other iters */
+	if (!gtk_tree_model_iter_children (model, pl_iter, &itdb_iter))
+	{   /* This indicates screwed up programming so we better cry
+	       out */
+	    g_return_val_if_reached (FALSE);
+	}
+
+	do
+	{
+	    gtk_tree_model_get (model, pl_iter,
+				PM_COLUMN_PLAYLIST, &pl,
+				-1);
+	    g_return_val_if_fail (pl, FALSE);
+	    if (pl == playlist)
+	    {
+		return TRUE;
+	    }
+	} while (gtk_tree_model_iter_next (model, pl_iter));
+    }
+    return FALSE;
+}
+
+
+
+
 /* ---------------------------------------------------------------- */
 /* Section for playlist display                                     */
 /* ---------------------------------------------------------------- */
@@ -1010,6 +1120,10 @@ void pm_remove_track (Playlist *playlist, Track *track)
 	    if ((playlist == current_playlist) ||
 		itdb_playlist_is_mpl (current_playlist))
 	    {
+	    	if (prefs_get_int (KEY_DISPLAY_COVERART))
+	    {
+	  		coverart_track_changed (track, COVERART_REMOVE_SIGNAL);
+	    }
 		st_remove_track (track, 0);
 	    }
 	}
@@ -1024,37 +1138,32 @@ void pm_add_track (Playlist *playlist, Track *track, gboolean display)
     if (playlist == current_playlist)
     {
 	st_add_track (track, TRUE, display, 0); /* Add to first sort tab */
+    
+    	/* As with add_track above, only add to the playlist if it is the current one */
+    	if (prefs_get_int (KEY_DISPLAY_COVERART))
+	    {
+  			coverart_track_changed (track, COVERART_CREATE_SIGNAL);
+	    }
     }
 }
 
-/* Used by pm_name_changed() to find the playlist that
-   changed name. If found, emit a "row changed" signal to display the change */
-static gboolean sr_model_playlist_name_changed (GtkTreeModel *model,
-						GtkTreePath *path,
-						GtkTreeIter *iter,
-						gpointer data)
-{
-  Playlist *playlist=NULL;
-
-  gtk_tree_model_get (model, iter, PM_COLUMN_PLAYLIST, &playlist, -1);
-  if(playlist == (Playlist *)data) {
-    gtk_tree_model_row_changed (model, path, iter);
-    return TRUE;
-  }
-  return FALSE;
-}
-
-
 /* One of the playlist names has changed (this happens when the
    iTunesDB is read */
-void pm_name_changed (Playlist *pl)
+void pm_itdb_name_changed (iTunesDB *itdb)
 {
-  GtkTreeModel *model = gtk_tree_view_get_model (playlist_treeview);
+  GtkTreeIter iter;
 
-  g_return_if_fail (pl);
-  g_return_if_fail (model);
+  g_return_if_fail (itdb);
 
-  gtk_tree_model_foreach (model, sr_model_playlist_name_changed, pl);
+  if (pm_get_iter_for_itdb (itdb, &iter))
+  {
+      GtkTreeModel *model;
+      GtkTreePath *path;
+      model = GTK_TREE_MODEL (gtk_tree_view_get_model (playlist_treeview));
+      path = gtk_tree_model_get_path (model, &iter);
+      gtk_tree_model_row_changed (model, path, &iter);
+      gtk_tree_path_free (path);
+  }
 }
 
 
@@ -1064,12 +1173,15 @@ void pm_name_changed (Playlist *pl)
 void pm_track_changed (Track *track)
 {
   if (!current_playlist) return;
+  
+  if (prefs_get_int (KEY_DISPLAY_COVERART))
+  	coverart_track_changed (track, COVERART_CHANGE_SIGNAL);
+  
   /* Check if track is member of current playlist */
   if (g_list_find (current_playlist->members, track))
       st_track_changed (track, FALSE, 0);
 }
 
-
 /* Add playlist to the playlist model */
 /* If @position = -1: append to end */
 /* If @position >=0: insert at that position (count starts with MPL as
@@ -1081,6 +1193,7 @@ void pm_add_playlist (Playlist *playlist, gint pos)
   GtkTreeIter iter;
   GtkTreeModel *model;
   GtkTreeSelection *selection;
+  iTunesDB *itdb;
 
   g_return_if_fail (playlist_treeview);
   g_return_if_fail (playlist);
@@ -1091,36 +1204,28 @@ void pm_add_playlist (Playlist *playlist, gint pos)
   if (itdb_playlist_is_mpl (playlist))
   {   /* MPLs are always added top-level */
       mpl = NULL;
+      itdb = playlist->itdb;
   }
   else
   {   /* We need to find the iter with the mpl in it */
-      if (gtk_tree_model_get_iter_first (model, &mpl_iter))
+      if (pm_get_iter_for_itdb (playlist->itdb, &mpl_iter))
       {
-	  do
-	  {
-	      Playlist *pl;
-	      gtk_tree_model_get (model, &mpl_iter,
-				  PM_COLUMN_PLAYLIST, &pl, -1);
-	      g_return_if_fail (pl);
-	      if (pl->itdb == playlist->itdb)
-	      {
-		  mpl = &mpl_iter;
-	      }
-	  } while ((mpl == NULL) &&
-		   gtk_tree_model_iter_next (model, &mpl_iter));
+	  mpl = &mpl_iter;
       }
-      if (!mpl)
+      else
       {
 	  g_warning ("Programming error: need to add mpl before adding normal playlists.\n");
       }
       /* reduce position by one because the MPL is not included in the
 	 tree model's count */
       --pos;
+      itdb = NULL;
   }
 
   gtk_tree_store_insert (GTK_TREE_STORE (model), &iter, mpl, pos);
 
   gtk_tree_store_set (GTK_TREE_STORE (model), &iter,
+		      PM_COLUMN_ITDB, itdb,
 		      PM_COLUMN_PLAYLIST, playlist,
 		      -1);
 
@@ -1146,22 +1251,6 @@ void pm_add_playlist (Playlist *playlist, gint pos)
 
 
 
-    /* former inline function: used by pm_remove_playlist() */
-    static gboolean pm_delete_playlist_fe (GtkTreeModel *model,
-					   GtkTreePath *path,
-					   GtkTreeIter *iter,
-					   gpointer data)
-	{
-	    Playlist *playlist=NULL;
-	    
-	    gtk_tree_model_get (model, iter, PM_COLUMN_PLAYLIST, &playlist, -1);
-	    if(playlist == (Playlist *)data) {
-		gtk_tree_store_remove (GTK_TREE_STORE (model), iter);
-		return TRUE;
-	    }
-	    return FALSE;
-	}
-
 /* Remove "playlist" from the display model. 
    "select": TRUE: a new playlist is selected
              FALSE: no selection is taking place
@@ -1170,7 +1259,7 @@ void pm_remove_playlist (Playlist *playlist, gboolean select)
 {
     GtkTreeModel *model;
     gboolean have_iter = FALSE;
-    GtkTreeIter i;
+    GtkTreeIter select_iter, delete_iter;
     GtkTreeSelection *ts = NULL;
 
   g_return_if_fail (playlist);
@@ -1192,10 +1281,10 @@ void pm_remove_playlist (Playlist *playlist, gboolean select)
   if (select && (current_playlist == playlist))
   {   /* We are about to delete the currently selected
 	 playlist. Try to select the next. */
-      if (gtk_tree_selection_get_selected (ts, NULL, &i))
+      if (gtk_tree_selection_get_selected (ts, NULL, &select_iter))
       {
-	  GtkTreePath *path = gtk_tree_model_get_path (model, &i);
-	  if(gtk_tree_model_iter_next (model, &i))
+	  GtkTreePath *path = gtk_tree_model_get_path (model, &select_iter);
+	  if(gtk_tree_model_iter_next (model, &select_iter))
 	  {
 	      have_iter = TRUE;
 	  }
@@ -1203,7 +1292,7 @@ void pm_remove_playlist (Playlist *playlist, gboolean select)
 	  {   /* no next iter -- try previous iter */
 	      if (gtk_tree_path_prev (path))
 	      {   /* OK -- make iter from it */
-		  gtk_tree_model_get_iter (model, &i, path);
+		  gtk_tree_model_get_iter (model, &select_iter, path);
 		  have_iter = TRUE;
 	      }
 	  }
@@ -1211,10 +1300,13 @@ void pm_remove_playlist (Playlist *playlist, gboolean select)
       }
   }
 
-  /* find the pl and delete it */
-  gtk_tree_model_foreach (model, pm_delete_playlist_fe, playlist);
+  if (pm_get_iter_for_playlist (playlist, &delete_iter))
+  {
+      gtk_tree_store_remove (GTK_TREE_STORE (model), &delete_iter);
+  }
+
   /* select our new iter !!! */
-  if (have_iter && select)   gtk_tree_selection_select_iter(ts, &i);
+  if (have_iter && select)   gtk_tree_selection_select_iter(ts, &select_iter);
 }
 
 
@@ -1249,81 +1341,43 @@ void pm_remove_all_playlists (gboolean clear_sort)
 }
 
 
-    /* former inline function: used by pm_select_playlist */
-    static gboolean pm_select_playlist_fe (GtkTreeModel *model,
-					   GtkTreePath *path,
-					   GtkTreeIter *iter,
-					   gpointer data)
-	{
-	    Playlist *playlist=NULL;
-
-	    gtk_tree_model_get (model, iter,
-				PM_COLUMN_PLAYLIST, &playlist, -1);
-	    if(playlist == data)
-	    {
-		GtkTreeSelection *ts = gtk_tree_view_get_selection (
-		    playlist_treeview);
-		gtk_tree_selection_select_iter (ts, iter);
-		return TRUE;
-	    }
-	    return FALSE;
-	}
-
 /* Select specified playlist */
 void pm_select_playlist (Playlist *playlist)
 {
-    GtkTreeModel *model;
+    GtkTreeIter iter;
 
     g_return_if_fail (playlist_treeview);
     g_return_if_fail (playlist);
-    model = gtk_tree_view_get_model (playlist_treeview);
-    g_return_if_fail (model);
 
-    /* find the pl and select it */
-    gtk_tree_model_foreach (model, pm_select_playlist_fe,
-			    playlist);
+    if (pm_get_iter_for_playlist (playlist, &iter))
+    {
+	GtkTreeSelection *ts;
+	ts = gtk_tree_view_get_selection (playlist_treeview);
+	gtk_tree_selection_select_iter (ts, &iter);
+    }
 }
 
 
-    /* former inline function: used by pm_unselect_playlist */
-    static gboolean pm_unselect_playlist_fe (GtkTreeModel *model,
-					     GtkTreePath *path,
-					     GtkTreeIter *iter,
-					     gpointer data)
-	{
-	    Playlist *playlist=NULL;
-
-	    gtk_tree_model_get (model, iter,
-				PM_COLUMN_PLAYLIST, &playlist, -1);
-	    if(playlist == data)
-	    {
-		GtkTreeSelection *ts = gtk_tree_view_get_selection (
-		    playlist_treeview);
-		gtk_tree_selection_unselect_iter (ts, iter);
-		return TRUE;
-	    }
-	    return FALSE;
-	}
-
 /* Unselect specified playlist */
 void pm_unselect_playlist (Playlist *playlist)
 {
-    GtkTreeModel *model;
+    GtkTreeIter iter;
 
     g_return_if_fail (playlist_treeview);
     g_return_if_fail (playlist);
-    model = gtk_tree_view_get_model (playlist_treeview);
-    g_return_if_fail (model);
 
-    /* find the pl and unselect it */
-    gtk_tree_model_foreach (model, pm_unselect_playlist_fe,
-			    playlist);
+    if (pm_get_iter_for_playlist (playlist, &iter))
+    {
+	GtkTreeSelection *ts;
+	ts = gtk_tree_view_get_selection (playlist_treeview);
+	gtk_tree_selection_unselect_iter (ts, &iter);
+    }
 }
 
 
-static void pm_selection_changed_cb (gpointer user_data1, gpointer user_data2)
+static void pm_selection_changed_cb (GtkTreeSelection *selection,
+				     gpointer user_data2)
 {
-  GtkTreeSelection *selection = (GtkTreeSelection *)user_data1;
   GtkTreeModel *model;
   GtkTreeIter  iter;
   Playlist *new_playlist = NULL;
@@ -1335,6 +1389,11 @@ static void pm_selection_changed_cb (gpointer user_data1, gpointer user_data2)
 	  time.tv_sec % 3600, time.tv_usec);
 #endif
 
+	/* Avoid track selection errors on coverart while enacting a change
+	 * in playlist
+	 */
+	 coverart_block_change (TRUE);
+  
   if (gtk_tree_selection_get_selected (selection, &model, &iter) == FALSE)
   {  /* no selection -> reset sort tabs */
       st_init (-1, 0);
@@ -1365,50 +1424,27 @@ static void pm_selection_changed_cb (gpointer user_data1, gpointer user_data2)
 	  itdb_spl_update (new_playlist);
       if (new_playlist->members)
       {
-	  GTimeVal time;
-	  float max_count = REFRESH_INIT_COUNT;
-	  gint count = max_count - 1;
-	  float ms;
 	  GList *gl;
 
-	  if (!prefs_get_int("block_display"))
-	  {
-	      block_selection (-1);
-	      g_get_current_time (&time);
-	  }
 	  st_enable_disable_view_sort (0, FALSE);
 	  for (gl=new_playlist->members; gl; gl=gl->next)
 	  { /* add all tracks to sort tab 0 */
 	      Track *track = gl->data;
-	      if (stop_add == -1)  break;
 	      st_add_track (track, FALSE, TRUE, 0);
-	      --count;
-	      if ((count < 0) && !prefs_get_int("block_display"))
-	      {
-		  gtkpod_tracks_statusbar_update();
-		  while (gtk_events_pending ())       gtk_main_iteration ();
-		  ms = get_ms_since (&time, TRUE);
-		  /* first time takes significantly longer, so we adjust
-		     the max_count */
-		  if (max_count == REFRESH_INIT_COUNT) max_count *= 2.5;
-		  /* average the new and the old max_count */
-		  max_count *= (1 + 2 * REFRESH_MS / ms) / 3;
-		  count = max_count - 1;
-#if DEBUG_TIMING
-		  printf("pm_s_c ms: %f mc: %f\n", ms, max_count);
-#endif
-	      }
 	  }
 	  st_enable_disable_view_sort (0, TRUE);
-	  if (stop_add != -1) st_add_track (NULL, TRUE, TRUE, 0);
-	  if (!prefs_get_int("block_display"))
-	  {
-	      while (gtk_events_pending ())	      gtk_main_iteration ();
-	      release_selection (-1);
-	  }
+	  st_add_track (NULL, TRUE, TRUE, 0);
       }
       gtkpod_tracks_statusbar_update();
   }
+  
+  /* Reallow the coverart selection update */
+  coverart_block_change (FALSE);
+  /* Set the coverart display based on the selected playlist */
+  coverart_set_images(TRUE);
+	 
+  space_data_update ();
+    
 #if DEBUG_TIMING
   g_get_current_time (&time);
   printf ("pm_selection_changed_cb exit:  %ld.%06ld sec\n",
@@ -1420,16 +1456,11 @@ static void pm_selection_changed_cb (gpointer user_data1, gpointer user_data2)
 
 /* Callback function called when the selection
    of the playlist view has changed */
-/* Instead of handling the selection directly, we add a
-   "callback". Currently running display updates will be stopped
-   before the pm_selection_changed_cb is actually called */
 static void pm_selection_changed (GtkTreeSelection *selection,
 				  gpointer user_data)
 {
-    space_data_update ();
     if (!pm_selection_blocked)
-	add_selection_callback (-1, pm_selection_changed_cb,
-				(gpointer)selection, user_data);
+	pm_selection_changed_cb (selection, user_data);
 }
 
 
@@ -1476,16 +1507,16 @@ void pm_add_itdb (iTunesDB *itdb, gint pos)
 
     for (gl_pl=itdb->playlists; gl_pl; gl_pl=gl_pl->next)
     {
-	Playlist *pl = gl_pl->data;
-	g_return_if_fail (pl);
-	if (itdb_playlist_is_mpl (pl))     pm_add_playlist (pl, pos);
-	else                               pm_add_playlist (pl, -1);
+			Playlist *pl = gl_pl->data;
+			g_return_if_fail (pl);
+			if (itdb_playlist_is_mpl (pl))     pm_add_playlist (pl, pos);
+			else                               pm_add_playlist (pl, -1);
     }
 }
 
 
 /* Helper function: add all playlists to playlist model */
-void pm_add_all_playlists (void)
+void pm_add_all_itdbs (void)
 {
     GList *gl_itdb;    
     struct itdbs_head *itdbs_head;
@@ -1503,49 +1534,98 @@ void pm_add_all_playlists (void)
 }
 
 
-/* Return path of playlist @pl. After use the return value must be
- * freed by calling gtk_tree_path_free() */
-GtkTreePath *pm_get_path (Playlist *pl)
+/* Return GtkTreePath for playlist @playlist. The returned path must be
+   freed using gtk_tree_path_free() after it is no needed any more */
+static GtkTreePath *pm_get_path_for_playlist (Playlist *playlist)
 {
-    struct userdata
-    {
-	Playlist *pl;
-	GtkTreePath *path;
-    };
-    GtkTreeModel *model;
-    struct userdata userdata;
-    gboolean pm_get_path_fe (GtkTreeModel *model,
-				    GtkTreePath *path,
-				    GtkTreeIter *iter,
-				    gpointer data)
-	{
-	    struct userdata *ud = data;
-	    Playlist *pl;
+    GtkTreeIter iter;
 
-	    gtk_tree_model_get (model, iter,
-				PM_COLUMN_PLAYLIST, &pl, -1);
-	    if(pl == ud->pl)
-	    {
-		ud->path = gtk_tree_model_get_path (model, iter);
-		return TRUE;
-	    }
-	    return FALSE;
-	}
     g_return_val_if_fail (playlist_treeview, NULL);
-    g_return_val_if_fail (pl, NULL);
-    model = gtk_tree_view_get_model (playlist_treeview);
-    g_return_val_if_fail (model, NULL);
+    g_return_val_if_fail (playlist, NULL);
 
-    userdata.pl = pl;
-    userdata.path = NULL;
-
-    /* find the pl and fill in path */
-    gtk_tree_model_foreach (model, pm_get_path_fe, &userdata);
-
-    return userdata.path;
+    if (pm_get_iter_for_playlist (playlist, &iter))
+    {
+	GtkTreeModel *model;
+	model = gtk_tree_view_get_model (playlist_treeview);
+	return gtk_tree_model_get_path (model, &iter);
+    }
+    return NULL;
 }
 
 
+/* Return GtkTreePath for repository @itdb. The returned path must be
+   freed using gtk_tree_path_free() after it is no needed any more */
+GtkTreePath *pm_get_path_for_itdb (iTunesDB *itdb)
+{
+    GtkTreeIter iter;
+
+    g_return_val_if_fail (playlist_treeview, NULL);
+    g_return_val_if_fail (itdb, NULL);
+
+    if (pm_get_iter_for_itdb (itdb, &iter))
+    {
+	GtkTreeModel *model;
+	model = gtk_tree_view_get_model (playlist_treeview);
+	return gtk_tree_model_get_path (model, &iter);
+    }
+    return NULL;
+}
+
+
+/* Return position of repository @itdb */
+gint pm_get_position_for_itdb (iTunesDB *itdb)
+{
+    GtkTreePath *path;
+    gint position = -1;
+
+    g_return_val_if_fail (playlist_treeview, -1);
+    g_return_val_if_fail (itdb, -1);
+
+    path = pm_get_path_for_itdb (itdb);
+
+    if (path)
+    {
+	gint *indices = gtk_tree_path_get_indices (path);
+	if (indices)
+	{
+	    position = indices[0];
+	}
+	gtk_tree_path_free (path);
+    }
+    return position;
+}
+
+
+/* Return position of repository @itdb */
+static gint pm_get_position_for_playlist (Playlist *playlist)
+{
+    GtkTreePath *path;
+    gint position = -1;
+
+    g_return_val_if_fail (playlist_treeview, -1);
+    g_return_val_if_fail (playlist, -1);
+
+    path = pm_get_path_for_playlist (playlist);
+
+    if (path)
+    {
+	/* get position of current path */
+	if (gtk_tree_path_get_depth (path) == 1)
+	{   /* MPL */
+	    position = 0;
+	}
+	else
+	{
+	    gint *indices = gtk_tree_path_get_indices (path);
+	    /* need to add 1 because MPL is one level higher and not
+	       counted */
+	    position = indices[1] + 1;
+	}
+	gtk_tree_path_free (path);
+    }
+    return position;
+}
+
 
 /* "unsort" the playlist view without causing the sort tabs to be
    touched. */
@@ -1563,7 +1643,7 @@ static void pm_unsort ()
     pm_set_selected_playlist (cur_pl);
 
     /* add playlists back to model (without selecting) */
-    pm_add_all_playlists ();
+    pm_add_all_itdbs ();
 
     pm_selection_blocked = FALSE;
     /* reset sort counter */
@@ -2036,6 +2116,10 @@ pm_button_press (GtkWidget *w, GdkEventButton *e, gpointer data)
 					       e->x, e->y, NULL);
 	if (cell_nr == 0)
 	{
+	    /* don't accept clicks while widgets are blocked -- this
+	       might cause a crash (e.g. when we click the 'Eject
+	       iPod' symbol while we are ejecting it already) */
+	    if (widgets_blocked) return FALSE;
 	    /* */
 	    model= gtk_tree_view_get_model (GTK_TREE_VIEW (w));
 	    gtk_tree_view_get_path_at_pos (GTK_TREE_VIEW(w),
@@ -2167,7 +2251,7 @@ void pm_create_treeview (void)
   gtk_container_add (GTK_CONTAINER (playlist_window), tree);
 
   /* create model */
-  model =   gtk_tree_store_new (PM_NUM_COLUMNS, G_TYPE_POINTER);
+  model =   gtk_tree_store_new (PM_NUM_COLUMNS, G_TYPE_POINTER, G_TYPE_POINTER);
 
   /* set tree model */
   gtk_tree_view_set_model (playlist_treeview, GTK_TREE_MODEL (model));
@@ -2184,7 +2268,7 @@ void pm_create_treeview (void)
 		       pm_drag_types, TGNR (pm_drag_types),
 		       GDK_ACTION_COPY|GDK_ACTION_MOVE);
   gtk_drag_dest_set (GTK_WIDGET (playlist_treeview),
-		     GTK_DEST_DEFAULT_MOTION | GTK_DEST_DEFAULT_HIGHLIGHT,
+		     GTK_DEST_DEFAULT_HIGHLIGHT,
 		     pm_drop_types, TGNR (pm_drop_types),
 		     GDK_ACTION_COPY|GDK_ACTION_MOVE);
 
