@@ -1,4 +1,4 @@
-/* Time-stamp: <2006-09-24 12:52:49 jcs>
+/* Time-stamp: <2007-06-26 00:39:11 jcs>
 |
 |  Copyright (C) 2002-2005 Jorg Schuler <jcsjcs at users sourceforge net>
 |  Part of the gtkpod project.
@@ -24,7 +24,7 @@
 |
 |  This product is not supported/written/published by Apple!
 |
-|  $Id: mp3file.c,v 1.71 2006/09/24 03:57:10 jcsjcs Exp $
+|  $Id: mp3file.c 1591 2007-06-26 03:33:25Z tmzullinger $
 */
 
 
@@ -53,6 +53,7 @@
  */
 typedef struct _File_Tag File_Tag;
 typedef struct _GainData GainData;
+typedef struct _GaplessData GaplessData;
 
 struct _File_Tag
 {
@@ -66,15 +67,20 @@ struct _File_Tag
     gchar *comment;        /* Comment */
     gchar *composer;	   /* Composer */
     guint32 songlen;       /* Length of file in ms */
-    /* CD/disc number handling */
     gchar *cdnostring;    /* Position of disc in the album */
     gchar *cdno_total;    /* The number of discs in the album (ex: 1/2) */
     gchar *compilation;   /* The track is a member of a compilation */
     gchar *podcasturl;    /* The following are mainly used for podcasts */
+    gchar *sort_artist;
+    gchar *sort_title;
+    gchar *sort_album;
+    gchar *sort_albumartist;
+    gchar *sort_composer;
     gchar *description;
     gchar *podcastrss;
     gchar *time_released;
     gchar *subtitle;
+    gchar *BPM;           /* beats per minute */
     gchar *lyrics;        /* does not appear to be the full lyrics --
 			     only used to set the flag 'lyrics_flag'
 			     of the Track structure */
@@ -93,6 +99,13 @@ struct _GainData
   gboolean audiophile_gain_set;/* has the audiophile gain been set?  */
 };
 
+struct _GaplessData
+{
+    guint32 pregap;       /* number of pregap samples */
+    guint64 samplecount;  /* number of actual music samples */
+    guint32 postgap;      /* number of postgap samples */
+    guint32 gapless_data; /* number of bytes from the first sync frame to the 8th to last frame */
+};
 
 /* This code is taken from the mp3info code. Only the code needed for
  * the playlength calculation has been extracted */
@@ -965,7 +978,8 @@ static guint get_track_time (gchar *path)
 
 static const gchar* id3_get_binary (struct id3_tag *tag,
 				    char *frame_name,
-				    id3_length_t *len)
+				    id3_length_t *len,
+				    int index)
 {
     const id3_byte_t *binary = NULL;
     struct id3_frame *frame;
@@ -975,7 +989,7 @@ static const gchar* id3_get_binary (struct id3_tag *tag,
 
     *len = 0;
 
-    frame = id3_tag_findframe (tag, frame_name, 0);
+    frame = id3_tag_findframe (tag, frame_name, index);
 #if LOCALDEBUG
     printf ("frame: %p\n", frame); 
 #endif
@@ -983,7 +997,12 @@ static const gchar* id3_get_binary (struct id3_tag *tag,
     if (!frame) return NULL;
 
 #if LOCALDEBUG
-    printf (" nfields: %d\n", frame->nfields); 
+    printf (" nfields: %d\n", frame->nfields);
+    if (strncmp (frame_name, "APIC",  4) == 0)
+    {
+	field = id3_frame_field (frame, 2);
+	printf (" picture type: %ld\n", field->number.value);
+    }
 #endif
 
 
@@ -1150,7 +1169,7 @@ static gchar* id3_get_string (struct id3_tag *tag, char *frame_name)
 
     if (!string) return NULL;
 
-    if (frame_name == ID3_FRAME_GENRE)
+    if (strcmp (frame_name, ID3_FRAME_GENRE) == 0)
        string = id3_genre_name (string);
 
     if (encoding == ID3_FIELD_TEXTENCODING_ISO_8859_1)
@@ -1200,7 +1219,7 @@ static void id3_set_string (struct id3_tag *tag,
     field = id3_frame_field (frame, 0);
     id3_field_settextencoding(field, encoding);
    
-    if (frame_name == ID3_FRAME_COMMENT)
+    if (strcmp (frame_name, ID3_FRAME_COMMENT) == 0)
     {
 	field = id3_frame_field (frame, 3);
 	field->type = ID3_FIELD_TYPE_STRINGFULL;
@@ -1215,7 +1234,7 @@ static void id3_set_string (struct id3_tag *tag,
     /* maybe could be optimized see
        http://www.mars.org/mailman/public/mad-dev/2002-October/000739.html
     */
-    if (frame_name == ID3_FRAME_GENRE)
+    if (strcmp (frame_name, ID3_FRAME_GENRE) == 0)
     {
 	id3_ucs4_t *tmp_ucs4 = id3_utf8_ucs4duplicate ((id3_utf8_t *)data);
 	int index = id3_genre_number (tmp_ucs4);
@@ -1265,7 +1284,7 @@ static void id3_set_string (struct id3_tag *tag,
 	}
     }
 
-    if (frame_name == ID3_FRAME_COMMENT)
+    if (strcmp (frame_name, ID3_FRAME_COMMENT) == 0)
 	res = id3_field_setfullstring (field, ucs4);
     else
 	res = id3_field_setstrings (field, 1, &ucs4);
@@ -1306,9 +1325,29 @@ static gboolean id3_apic_read (gchar *filename,
     if ((id3tag = id3_file_tag(id3file)))
     {
 	id3_length_t len;
-	const guchar *coverart;
+	const guchar *coverart = NULL;
+	int i;
+	struct id3_frame *frame;
 
-	coverart = id3_get_binary (id3tag, "APIC", &len);
+	/* Loop through APIC tags and set coverart.  The picture type should be
+	 * 3 -- Cover (front), but iTunes has been known to use 0 -- Other. */
+	for (i = 0; (frame = id3_tag_findframe(id3tag, "APIC", i)) != NULL; i++)
+	{
+	    union id3_field *field = id3_frame_field (frame, 2);
+	    int pictype = field->number.value;
+/*	    printf ("%s: found apic type %d\n", filename, pictype);*/
+
+	    /* We'll prefer type 3 (cover) over type 0 (other) */
+	    if (pictype == 3)
+	    {
+		coverart = id3_get_binary (id3tag, "APIC", &len, i);
+		break;
+	    }
+	    if ((pictype == 0) && !coverart)
+	    {
+		coverart = id3_get_binary (id3tag, "APIC", &len, i);
+	    }
+	}
 
 	if (coverart)
 	{   /* I guess iTunes is doing something wrong -- the
@@ -1397,6 +1436,12 @@ gboolean id3_tag_read (gchar *filename, File_Tag *tag)
 	tag->podcastrss = id3_get_string (id3tag, "YWFD");
 	tag->description = id3_get_string (id3tag, "YTDS");
 	tag->time_released = id3_get_string (id3tag, "YTDR");
+	tag->BPM = id3_get_string (id3tag, "TBPM");
+	tag->sort_artist = id3_get_string (id3tag, "TSOP");
+	tag->sort_album = id3_get_string (id3tag, "TSOA");
+	tag->sort_title = id3_get_string (id3tag, "TSOT");
+	tag->sort_albumartist = id3_get_string (id3tag, "TSO2");
+	tag->sort_composer = id3_get_string (id3tag, "TSOC");
 
 	string = id3_get_string (id3tag, "TLEN");
 	if (string)
@@ -1555,6 +1600,11 @@ gboolean mp3_write_file_info (gchar *filename, Track *track)
 	id3_set_string (id3tag, ID3_FRAME_GENRE, track->genre, encoding);
 	id3_set_string (id3tag, ID3_FRAME_COMMENT, track->comment, encoding);
 	id3_set_string (id3tag, "TIT3", track->subtitle, encoding);
+	id3_set_string (id3tag, "TSOP", track->sort_artist, encoding);
+	id3_set_string (id3tag, "TSOA", track->sort_album, encoding);
+	id3_set_string (id3tag, "TSOT", track->sort_title, encoding);
+	id3_set_string (id3tag, "TSO2", track->sort_albumartist, encoding);
+	id3_set_string (id3tag, "TSOC", track->sort_composer, encoding);
 
 	set_uncommon_tag (id3tag, "YTID", track->podcasturl, encoding);
 	set_uncommon_tag (id3tag, "YTDS", track->description, encoding);
@@ -1566,6 +1616,10 @@ gboolean mp3_write_file_info (gchar *filename, Track *track)
 	id3_set_string(id3tag, ID3_FRAME_YEAR, string1, encoding);
 	g_free(string1);
 
+	string1 = g_strdup_printf("%d", track->BPM);
+	id3_set_string(id3tag, "TBPM", string1, encoding);
+	g_free(string1);
+
 	if (track->tracks)
 	    string1 = g_strdup_printf ("%d/%d",
 				       track->track_nr, track->tracks);
@@ -2093,7 +2147,7 @@ rg_fail:
  *
  * Returns TRUE if the soundcheck field could be set.
  */
-gboolean mp3_read_soundcheck (gchar *path, Track *track) 
+gboolean mp3_read_soundcheck (gchar *path, Track *track)
 {
     GainData gd;
 
@@ -2124,6 +2178,251 @@ gboolean mp3_read_soundcheck (gchar *path, Track *track)
 
 
 
+
+
+
+/* mp3 slot size in bytes */
+int slotsize[3] = {4,1,1}; /* layer 1, layer 2, layer 3 */
+
+int samplesperframe[2][3] = {
+  { /* MPEG 2.0 */
+    384,1152,576 /* layer 1, layer 2, layer 3 */
+  },
+
+  { /* MPEG 1.0 */
+    384,1152,1152 /* layer 1, layer 2, layer 3 */
+  }
+};
+
+
+/* 
+ * mp3_get_track_lame_gapless - read the specified file and scan for LAME Tag
+ * gapless information.
+ *
+ * @path: localtion of the file
+ * @track: structure holding track information
+ *
+ * TODO: Split off non-LAME stuff (samplecount, gapless_data) to a separate function since it's generic
+ */
+gboolean mp3_get_track_lame_gapless (gchar *path, GaplessData *gd)
+{
+    FILE *file = NULL;
+    char buf[4], version[5];
+    unsigned char ubuf[4];
+    int sideinfo;
+    int i;
+
+    g_return_val_if_fail (gd, FALSE);
+
+    if (!path)
+	goto gp_fail;
+
+    file = fopen (path, "rb");
+
+    if (!file)
+	goto gp_fail;
+
+    /* use get_first_header() to seek to the first mp3 header */
+    MP3Info *mp3i = NULL;
+    mp3i = g_malloc0 (sizeof (MP3Info));
+    mp3i->filename = path;
+    mp3i->file = file;
+    get_mp3_info (mp3i);
+    get_first_header (mp3i, 0);
+
+    int xing_header_offset = ftell (file);
+
+    MP3Header h;
+    if (!get_header (file, &h))
+	goto gp_fail;
+
+    int mysamplesperframe = samplesperframe[h.version & 1][3 - h.layer];
+
+    /* Determine offset of Xing header based on sideinfo size */
+    if (h.version & 0x1)
+    {
+	sideinfo = (h.mode & 0x2) ?
+	    SIDEINFO_MPEG1_MONO : SIDEINFO_MPEG1_MULTI;
+    }
+    else
+    {
+	sideinfo = (h.mode & 0x2) ?
+	    SIDEINFO_MPEG2_MONO : SIDEINFO_MPEG2_MULTI;
+    }
+
+    if (fseek (file, sideinfo, SEEK_CUR) ||
+	(fread (&buf[0], 1, 4, file) != 4))
+	goto gp_fail;
+
+    /* Is this really a Xing or Info Header? 
+     * FIXME: Apparently (according to madplay sources) there is a different
+     * possible location for the Xing header ("due to an unfortunate
+     * historical event"). I do not thing we need to care though since
+     * ReplayGain information is only contained in recent files.  */
+    if (strncmp (buf, "Xing", 4) && strncmp (buf, "Info", 4))
+	goto gp_fail;
+
+    /* Determine the offset of the LAME tag based on contents of the Xing header */
+    int flags;
+    fread (&flags, 4, 1, file);
+    int toskip = 0;
+    if (flags | 0x1)
+    {				/* frames field is set */
+	toskip += 4;
+    }
+    if (flags | 0x2)
+    {				/* bytes field is set */
+	toskip += 4;
+    }
+    if (flags | 0x4)
+    {				/* TOC field is set */
+	toskip += 100;
+    }
+    if (flags | 0x8)
+    {				/* quality field is set */
+	toskip += 4;
+    }
+
+    /* Check for LAME Tag */
+    if (fseek (file, toskip, SEEK_CUR) || (fread (&buf[0], 1, 4, file) != 4))
+	goto gp_fail;
+    if (strncmp (buf, "LAME", 4))
+	goto gp_fail;
+
+    /* Check LAME Version */
+    if (fread (version, 1, 5, file) != 5)
+	goto gp_fail;
+
+    /* XXX skip old LAME versions, or just assume that pre/postgap
+     * turn out zeros anyway, or check the CRC to vaidate the tag? */
+
+    gboolean cbr = FALSE;
+    if (fread (ubuf, 1, 1, file) != 1)
+	goto gp_fail;
+
+    if ((ubuf[0] & 0xf) == 0x1)
+	cbr = TRUE;
+
+    if (fseek (file, 0xB, SEEK_CUR) || (fread (ubuf, 1, 4, file) != 4))
+	goto gp_fail;
+
+    /* set pregap and postgap directly from LAME header */
+    gd->pregap = (ubuf[0] << 4) + (ubuf[1] >> 4);
+    gd->postgap = ((ubuf[1] & 0xf) << 8) + ubuf[2];
+
+    /* jump the end of the frame with the xing header */
+    if (fseek (file, xing_header_offset + frame_length (&h), SEEK_SET))
+	goto gp_fail;
+
+    /* counts bytes from the start of the 1st sync frame */
+    int totaldatasize = frame_length (&h);
+
+    /* keeps track of the last 8 frame sizes */
+    int lastframes[8];
+
+    /* counts number of music frames */
+    int totalframes = 0;
+
+    /* quickly parse the file, reading only frame headers */
+    int l = 0;
+    while ((l = get_header (file, &h)) != 0)
+    {
+	for (i = 7; i > 0; i--)
+	{
+	    lastframes[i] = lastframes[i - 1];
+	}
+	lastframes[0] = l;
+	totaldatasize += l;
+	totalframes++;
+
+	if (fseek (file, l - FRAME_HEADER_SIZE, SEEK_CUR))
+	    goto gp_fail;
+
+    }
+
+    int finaleight = 0;
+    for (i = 0; i < 8; i++)
+    {
+	finaleight += lastframes[i];
+    }
+
+    if (cbr)
+	totalframes++;
+
+    /* all but last eight frames */
+    gd->gapless_data = totaldatasize - finaleight;
+    /* total samples minus pre/postgap */
+    gd->samplecount = totalframes * mysamplesperframe - gd->pregap - gd->postgap;
+
+    return TRUE;
+
+
+  gp_fail:
+    if (file)
+	fclose (file);
+    return FALSE;
+
+}
+
+
+
+/** 
+ * mp3_read_gapless:
+ *
+ * try to read the gapless values from the LAME Tag and set
+ * the track's pregap, postgap, samplecount, and gapless_data fields
+ * accordingly.
+ *
+ * @path: location of the file
+ * @track: structure holding track information
+ *
+ * The function always rereads the data from the file.
+ *
+ * Returns TRUE if all four gapless fields could be
+ * set. etrack->tchanged is set to TRUE if data has been changed,
+ * FALSE otherwise.
+ */
+gboolean mp3_read_gapless (char *path, Track *track)
+{
+    GaplessData gd;
+    ExtraTrackData *etr;
+
+    g_return_val_if_fail (track, FALSE);
+
+    etr = track->userdata;
+
+    memset (&gd, 0, sizeof (GaplessData));
+
+    gd.pregap = 0;
+    gd.samplecount = 0;
+    gd.postgap = 0;
+    gd.gapless_data = 0;
+
+    mp3_get_track_lame_gapless (path, &gd);
+
+    etr->tchanged = FALSE;
+
+    if ((gd.pregap) && (gd.samplecount) && (gd.postgap) && (gd.gapless_data))
+    {
+	if ((track->pregap != gd.pregap) ||
+	    (track->samplecount != gd.samplecount) ||
+	    (track->postgap != gd.postgap) ||
+	    (track->gapless_data != gd.gapless_data) ||
+	    (track->gapless_track_flag == FALSE))
+	{
+	    etr->tchanged = TRUE;
+	    track->pregap = gd.pregap;
+	    track->samplecount = gd.samplecount;
+	    track->postgap = gd.postgap;
+	    track->gapless_data = gd.gapless_data;
+	    track->gapless_track_flag = TRUE;
+	}
+    }
+    return FALSE;
+}
+
+
+
 /* ----------------------------------------------------------------------
 
 	      From here starts original gtkpod code
@@ -2219,6 +2518,31 @@ Track *mp3_get_file_info (gchar *name)
 	    track->description = filetag.description;
 	}
 
+	if (filetag.sort_artist)
+	{
+	    track->sort_artist = filetag.sort_artist;
+	}
+
+	if (filetag.sort_title)
+	{
+	    track->sort_title = filetag.sort_title;
+	}
+
+	if (filetag.sort_album)
+	{
+	    track->sort_album = filetag.sort_album;
+	}
+
+	if (filetag.sort_albumartist)
+	{
+	    track->sort_albumartist = filetag.sort_albumartist;
+	}
+
+	if (filetag.sort_composer)
+	{
+	    track->sort_composer = filetag.sort_composer;
+	}
+
 	if (filetag.year == NULL)
 	{
 	    track->year = 0;
@@ -2279,6 +2603,16 @@ Track *mp3_get_file_info (gchar *name)
 	    g_free (filetag.compilation);
 	}
 
+	if (filetag.BPM == NULL)
+	{
+	    track->BPM = 0;
+	}
+	else
+	{
+	    track->BPM = atoi(filetag.BPM);
+	    g_free (filetag.BPM);
+	}
+
 	if (filetag.lyrics)
 	{
 	    track->lyrics_flag = 0x01;
@@ -2304,6 +2638,17 @@ Track *mp3_get_file_info (gchar *name)
 
     mp3_read_soundcheck (name, track);
 
+    mp3_read_gapless (name, track);
+
+#if LOCALDEBUG
+	printf("%s\n", name);
+    printf("\tpregap: %i\n", track->pregap);
+    printf("\tpostgap: %i\n", track->postgap);
+    printf("\tsamplecount: %li\n", track->samplecount);
+    printf("\tgaplessdata: %i\n", track->gapless_data);
+#endif
+
+
     /* Get additional info (play time and bitrate */
     if (mp3i)
     {
