@@ -1,5 +1,4 @@
-/* Time-stamp: <2006-09-24 20:49:38 jcs>
-|
+/*
 |  Copyright (C) 2002 Corey Donohoe <atmos at atmos.org>
 |  Copyright (C) 2002-2005 Jorg Schuler <jcsjcs at users sourceforge net>
 |  Part of the gtkpod project.
@@ -25,7 +24,7 @@
 |
 |  This product is not supported/written/published by Apple!
 |
-|  $Id: prefs_window.c,v 1.185 2006/09/24 12:16:26 jcsjcs Exp $
+|  $Id: prefs_window.c 1594 2007-06-26 04:02:23Z tmzullinger $
 */
 
 #include <stdio.h>
@@ -34,7 +33,8 @@
 #include "display_itdb.h"
 #include "info.h"
 #include "fileselection.h"
-#include "md5.h"
+#include "sha1.h"
+/*#include "md5.h"*/
 #include "misc.h"
 #include "misc_track.h"
 #include "prefs.h"
@@ -66,11 +66,20 @@ static void prefs_window_set_sort_tab_num (gint num);
 static void standard_toggle_toggled (GtkToggleButton *togglebutton,
 				     const gchar *key);
 
+static void on_convert_toggle_toggled (GtkToggleButton *togglebutton,
+				     gpointer not_used);
 
+static const gchar *convert_names[] =
+{
+    "convert_ogg",
+    "convert_flac",
+    "convert_m4a",
+    "convert_mp3",
+    "convert_wav",
+    NULL
+};
 
 /* Definition of path button names.
-   E.g. path_button_names[PATH_PLAY_ENQUEUE] is
-   "play_enqueue_path_button".
    path_fileselector_titles[] specifies the title for the file
    chooser. path_type[] specifies whether to browse for dirs or for
    files.
@@ -80,12 +89,14 @@ static const gchar *path_button_names[] =
     "play_now_path_button",
     "play_enqueue_path_button",
     "mp3gain_path_button",
-    "",
-    "",
+    "aacgain_path_button",
     "mserv_music_root_button",
     "mserv_trackinfo_root_button",
-    "",
-    "aacgain_path_button",
+    "path_conv_ogg_button",
+    "path_conv_flac_button",
+    "path_conv_m4a_button",
+    "path_conv_mp3_button",
+    "path_conv_wav_button",
     NULL
 };
 static const gchar *path_key_names[] =
@@ -93,12 +104,14 @@ static const gchar *path_key_names[] =
     "path_play_now",
     "path_play_enqueue",
     "path_mp3gain",
-    "",
-    "",
+    "aacgain_path",
     "path_mserv_music_root",
     "path_mserv_trackinfo_root",
-    "",
-    "aacgain_path",
+    "path_conv_ogg",
+    "path_conv_flac",
+    "path_conv_m4a",
+    "path_conv_mp3",
+    "path_conv_wav",
     NULL
 };
 const gchar *path_entry_names[] =
@@ -106,12 +119,14 @@ const gchar *path_entry_names[] =
     "play_now_path_entry",
     "play_enqueue_path_entry",
     "mp3gain_path_entry",
-    "",
-    "",
+    "aacgain_path_entry",
     "mserv_music_root_entry",
     "mserv_trackinfo_root_entry",
-    "",
-    "aacgain_path_entry",
+    "path_conv_ogg_entry",
+    "path_conv_flac_entry",
+    "path_conv_m4a_entry",
+    "path_conv_mp3_entry",
+    "path_conv_wav_entry",
     NULL
 };
 static const gchar *path_fileselector_titles[] =
@@ -119,12 +134,14 @@ static const gchar *path_fileselector_titles[] =
     N_("Please select command for 'Play Now'"),
     N_("Please select command for 'Enqueue'"),
     N_("Please select the mp3gain executable"),
-    "",
-    "",
+    N_("Please select the aacgain executable"),
     N_("Select the mserv music root directory"),
     N_("Select the mserv trackinfo root directory"),
-    "",
-    N_("Please select the aacgain executable"),
+    N_("Select the ogg/vorbis converter command"),
+    N_("Select the flac converter command"),
+    N_("Select the m4a converter command."),
+    N_("Select the mp3 converter command."),
+    N_("Select the wav converter command."),
     NULL
 };
 static const GtkFileChooserAction path_type[] =
@@ -133,15 +150,16 @@ static const GtkFileChooserAction path_type[] =
     GTK_FILE_CHOOSER_ACTION_OPEN,
     GTK_FILE_CHOOSER_ACTION_OPEN,
     GTK_FILE_CHOOSER_ACTION_OPEN,
-    GTK_FILE_CHOOSER_ACTION_OPEN,
     GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER, /* select folder */
     GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER,
     GTK_FILE_CHOOSER_ACTION_OPEN,
     GTK_FILE_CHOOSER_ACTION_OPEN,
+    GTK_FILE_CHOOSER_ACTION_OPEN,
+    GTK_FILE_CHOOSER_ACTION_OPEN,
+    GTK_FILE_CHOOSER_ACTION_OPEN,
     -1
 };
 
-
 static void on_cfg_st_autoselect_toggled (GtkToggleButton *togglebutton,
 					  gpointer         user_data)
 {
@@ -174,6 +192,8 @@ static void on_path_button_pressed (GtkButton *button, gpointer user_data)
 {
     gint i = GPOINTER_TO_INT (user_data);
     gchar *oldpath, *newpath;
+    gchar *fallback = NULL;
+    gchar *text = NULL;
 
     g_return_if_fail (temp_prefs);
 
@@ -183,15 +203,23 @@ static void on_path_button_pressed (GtkButton *button, gpointer user_data)
 	oldpath = prefs_get_string (path_key_names[i]);
     }
 
+    /* initialize fallback path with something reasonable */
+    if ((strcmp (path_key_names[i], "path_conv_ogg") == 0) ||
+	(strcmp (path_key_names[i], "path_conv_flac") == 0))
+    {
+	fallback = g_strdup (SCRIPTDIR);
+	text = g_markup_printf_escaped (_("<i>Have a look at the scripts provided in '%s'. If you write a new script or improve an existing one, please send it to jcsjcs at users.sourceforge.net for inclusion into the next release.</i>"), SCRIPTDIR);
+    }
+
     switch (path_type[i])
     {
     case GTK_FILE_CHOOSER_ACTION_OPEN:
 	/* script */
 	newpath = fileselection_select_script (
 	    oldpath,
-	    NULL,
+	    fallback,
 	    _(path_fileselector_titles[i]),
-	    NULL);
+	    text);
 	break;
     case GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER:
 	/* directory */
@@ -204,6 +232,8 @@ static void on_path_button_pressed (GtkButton *button, gpointer user_data)
 	g_return_if_reached ();
     }
     g_free (oldpath);
+    g_free (fallback);
+    g_free (text);
 
     if (newpath)
     {
@@ -262,6 +292,112 @@ void prefs_window_show_hide_tooltips (void)
     else                                     gtk_tooltips_disable (tt);
 }
 
+static void convert_table_set_children_initial_sensitivity (GtkTable *table)
+{
+    GList *list_item;
+    GtkTableChild *child;
+
+    list_item = table->children;
+    while (list_item) {
+        child = (GtkTableChild*)list_item->data;
+        gtk_widget_set_sensitive (child->widget, 
+                                     GTK_IS_TOGGLE_BUTTON (child->widget));
+        list_item = g_list_next (list_item);
+    }
+}
+
+static gint setup_visible_cols_buttons_sort (gconstpointer a, gconstpointer b)
+{
+    const gint *aa = a;
+    const gint *bb = b;
+
+    g_return_val_if_fail (a&&b, 0);
+
+    return g_utf8_collate (gettext (get_tm_string (*aa)),
+			   gettext (get_tm_string (*bb)));
+}
+
+/* Creates the toggle buttons to select the visible columns */
+static void setup_visible_cols_buttons ( GtkTooltips *tt)
+{
+    GArray *array;
+    GtkWidget *hbox;
+    GtkWidget *table;
+    gint i, rows, columns, r, c;
+
+    /* Sort the available colums alphabetically */
+    array = g_array_new (FALSE, FALSE, sizeof (gint));
+    for (i=0; i<TM_NUM_COLUMNS; ++i)
+    {
+	g_array_append_val (array, i);
+    }
+    g_array_sort (array, setup_visible_cols_buttons_sort);
+
+#if 0
+    for (i=0; i<TM_NUM_COLUMNS; ++i)
+    {
+	printf ("%s\n", gettext (get_tm_string (g_array_index (array, gint, i))));
+    }
+#endif
+
+    hbox = gtkpod_xml_get_widget (prefs_window_xml, "visible_cols_hbox");
+
+    /* how big must the table be to keep all column labels? */
+    columns = 5;
+    rows = (TM_NUM_COLUMNS + (columns - 1)) / columns;
+    table = gtk_table_new (rows, columns, TRUE);
+
+    for (r=0; r*columns < TM_NUM_COLUMNS; ++r)
+    {
+	for (c=0; c<columns; ++c)
+	{
+	    i = r*columns+c;
+	    if (i < TM_NUM_COLUMNS)
+	    {
+		GtkWidget *button;
+		gint j;
+		j = g_array_index (array, gint, i);
+		/* Create button */
+		button = gtk_check_button_new_with_label (
+		    gettext (get_tm_string (j)));
+		/* set active/inactive */
+		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button),
+					     prefs_get_int_index("col_visible", j));
+		/* set tooltip if available */
+		if (get_tm_tooltip (j))
+		{
+		    gtk_tooltips_set_tip (tt, button, 
+					  gettext (get_tm_tooltip (j)),
+					  NULL);
+		}
+		/* connect the signal handler */
+		g_signal_connect ((gpointer)button,
+				  "toggled",
+				  G_CALLBACK (on_cfg_col_visible_toggled),
+				  GUINT_TO_POINTER(j));
+		/* attach button to table */
+		gtk_table_attach (GTK_TABLE (table),
+				  button,
+				  c, c+1,
+				  r, r+1,
+				  GTK_FILL, GTK_SHRINK,
+				  0, 0);
+		/* show the button */
+		gtk_widget_show (button);
+	    }
+	}
+    }
+    /* Set table spacings */
+    gtk_table_set_row_spacings (GTK_TABLE (table), 4);
+    gtk_table_set_col_spacings (GTK_TABLE (table), 4);
+    /* Make table visible */
+    gtk_widget_show (table);
+    /* pack the table into the hbox */
+    gtk_box_pack_start (GTK_BOX (hbox), table, TRUE, TRUE, 0);
+
+    /* free memory */
+    g_array_free (array, TRUE);
+}
 
 
 /**
@@ -289,6 +425,8 @@ prefs_window_create (gint page)
 	"sync_confirm_dirs_toggle",
 	"sync_delete_tracks_toggle",
 	"sync_show_summary_toggle",
+	"file_convert_display_log_button",
+	"file_convert_background_transfer_button",
 	NULL
     };
     /* ... and corresponding keys */
@@ -296,6 +434,8 @@ prefs_window_create (gint page)
 	KEY_SYNC_CONFIRM_DIRS,
 	KEY_SYNC_DELETE_TRACKS,
 	KEY_SYNC_SHOW_SUMMARY,
+	FILE_CONVERT_DISPLAY_LOG,
+	FILE_CONVERT_BACKGROUND_TRANSFER,
 	NULL
     };
 
@@ -331,6 +471,11 @@ prefs_window_create (gint page)
 
     defx = prefs_get_int("size_prefs.x");
     defy = prefs_get_int("size_prefs.y");
+    if (defx == 0)
+	defx = 500;
+    if (defy == 0)
+	defy = 600;
+
     gtk_window_set_default_size (GTK_WINDOW (prefs_window), defx, defy);
 
     /* Code to add subscriptions list box */
@@ -359,9 +504,9 @@ prefs_window_create (gint page)
     w = gtkpod_xml_get_widget (prefs_window_xml, "charset_combo");
     charset_init_combo (GTK_COMBO (w));
     
-    w = gtkpod_xml_get_widget (prefs_window_xml, "cfg_md5tracks");
+    w = gtkpod_xml_get_widget (prefs_window_xml, "cfg_sha1tracks");
     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w),
-				 prefs_get_int("md5"));
+				 prefs_get_int("sha1"));
 
     w = gtkpod_xml_get_widget (prefs_window_xml, "cfg_update_existing");
     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w),
@@ -370,7 +515,7 @@ prefs_window_create (gint page)
     w = gtkpod_xml_get_widget (prefs_window_xml, "cfg_show_duplicates");
     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w),
 				 prefs_get_int("show_duplicates"));
-    if (!prefs_get_int("md5")) gtk_widget_set_sensitive (w, FALSE);
+    if (!prefs_get_int("sha1")) gtk_widget_set_sensitive (w, FALSE);
 
     w = gtkpod_xml_get_widget (prefs_window_xml, "cfg_show_updated");
     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w),
@@ -410,7 +555,7 @@ prefs_window_create (gint page)
     w = gtkpod_xml_get_widget (prefs_window_xml, "cfg_display_tooltips_prefs");
     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w),
 				 prefs_get_int("display_tooltips_prefs"));
-
+		
     w = gtkpod_xml_get_widget (prefs_window_xml, "cfg_multi_edit");
     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w),
 				 prefs_get_int("multi_edit"));
@@ -433,10 +578,6 @@ prefs_window_create (gint page)
     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w),
 				 prefs_get_int("update_charset"));
 
-    w = gtkpod_xml_get_widget (prefs_window_xml, "cfg_block_display");
-    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w),
-				 prefs_get_int("block_display"));
-
     w = gtkpod_xml_get_widget (prefs_window_xml, "cfg_id3_write");
     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w),
 				 prefs_get_int("id3_write"));
@@ -471,9 +612,24 @@ prefs_window_create (gint page)
     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w),
 				 prefs_get_int("delete_database"));
 
-    w = gtkpod_xml_get_widget (prefs_window_xml,  "cfg_autoimport");
-    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w),
-				 prefs_get_int("autoimport"));
+#if 0
+    /* last.fm -- disabled, we'll hide the prefs window */
+    int x = prefs_get_int("lastfm_active");
+    GtkWidget *u = gtkpod_xml_get_widget (prefs_window_xml, "lastfm_username");
+    GtkWidget *p = gtkpod_xml_get_widget (prefs_window_xml, "lastfm_password");
+    w = gtkpod_xml_get_widget (prefs_window_xml, "lastfm_active");
+
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w), x);
+    gtk_entry_set_text(GTK_ENTRY(u), prefs_get_string("lastfm_username"));
+    gtk_entry_set_text(GTK_ENTRY(p), prefs_get_string("lastfm_password"));
+    gtk_widget_set_sensitive (u, x);
+    gtk_widget_set_sensitive (p, x);
+# else
+    w = gtkpod_xml_get_widget (prefs_window_xml, "labelfm");
+    gtk_widget_hide(w);
+    w = gtkpod_xml_get_widget (prefs_window_xml, "scrolledwindowfm");
+    gtk_widget_hide(w);
+#endif
 
     w = gtkpod_xml_get_widget (prefs_window_xml, "autoselect_hbox");
     for (i=0; i<SORT_TAB_MAX; ++i)
@@ -585,34 +741,8 @@ prefs_window_create (gint page)
     }
     gtk_widget_set_sensitive (w, prefs_get_int("coverart_file"));
 
-    for (i=0; i<TM_NUM_COLUMNS; ++i)
-    {
-	buf = g_strdup_printf ("col_visible%d", i);
-	if((w = gtkpod_xml_get_widget (prefs_window_xml, buf)))
-	{
-	    /* set label */
-	    gtk_button_set_label (GTK_BUTTON (w),
-				  gettext (get_tm_string (i)));
-	    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w),
-					 prefs_get_int_index("col_visible", i));
-	    /* set tooltip if available */
-	    if (get_tm_tooltip (i))
-	    {
-		gtk_tooltips_set_tip (tt, w, 
-				      gettext (get_tm_tooltip (i)),
-				      NULL);
-	    }
-	    /* glade makes a "GTK_OBJECT (i)" which segfaults
-	       because "i" is not a GTK object. So we have to set
-	       up the signal handlers ourselves */
-	    g_signal_connect ((gpointer)w,
-			      "toggled",
-			      G_CALLBACK (on_cfg_col_visible_toggled),
-			      GUINT_TO_POINTER(i));
-	}
-	g_free (buf);
-    }
-    
+    setup_visible_cols_buttons (tt);
+
     w = gtkpod_xml_get_widget (prefs_window_xml, "cfg_write_extended");
     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w),
 				 prefs_get_int("write_extended_info"));
@@ -632,10 +762,6 @@ prefs_window_create (gint page)
 				     prefs_get_int("group_compilations"));
     }
 
-    w = gtkpod_xml_get_widget (prefs_window_xml, "cfg_tmp_disable_sort");
-    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w),
-				 prefs_get_int("tmp_disable_sort"));
-
     w = gtkpod_xml_get_widget (prefs_window_xml, "cfg_startup_messages");
     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w),
 				 prefs_get_int("startup_messages"));
@@ -673,9 +799,40 @@ prefs_window_create (gint page)
 	gtk_entry_set_text(GTK_ENTRY(w), buf);
 	g_free(buf);
     }
-    
+
+    w = gtkpod_xml_get_widget (prefs_window_xml, "convert_table");
+    convert_table_set_children_initial_sensitivity (GTK_TABLE (w));
+    for (i=0; convert_names[i]; ++i)
+    {
+         w = gtkpod_xml_get_widget (prefs_window_xml, convert_names[i]);
+         g_signal_connect ((gpointer)w,
+			      "toggled",
+			      G_CALLBACK (on_convert_toggle_toggled),
+			      NULL);
+         gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(w), prefs_get_int (convert_names[i]) );
+
+    }
+
+    w = gtkpod_xml_get_widget (prefs_window_xml, "file_convert_max_threads_num_spinbutton");
+    gtk_spin_button_set_value (GTK_SPIN_BUTTON(w),
+			       prefs_get_int (FILE_CONVERT_MAX_THREADS_NUM));
+
+    w = gtkpod_xml_get_widget (prefs_window_xml, "file_convert_maxdirsize_spinbutton");
+    gtk_spin_button_set_value (GTK_SPIN_BUTTON(w),
+			       prefs_get_double (FILE_CONVERT_MAXDIRSIZE));
+
+    w = gtkpod_xml_get_widget (prefs_window_xml, "file_convert_cachedir_entry");
+    buf = prefs_get_string (FILE_CONVERT_CACHEDIR);
+    if (buf)
+	gtk_entry_set_text (GTK_ENTRY(w), buf);
+    g_free (buf);
+
+
     prefs_window_show_hide_tooltips ();
     gtk_widget_show(prefs_window);
+
+
+
 }
 
 
@@ -693,9 +850,9 @@ prefs_window_set(void)
      if (temp_prefs_get_int_value(temp_prefs, "sort_tab_num", NULL))
       st_show_visible();
 
-     /* Set up/free md5 hash table if changed */
-     if (temp_prefs_get_int_value(temp_prefs, "md5", NULL))
-	 setup_md5();
+     /* Set up/free sha1 hash table if changed */
+     if (temp_prefs_get_int_value(temp_prefs, "sha1", NULL))
+	 setup_sha1();
    }
   
    /* Need this in case user reordered column order (we don't
@@ -706,6 +863,9 @@ prefs_window_set(void)
    st_show_visible();
    display_show_hide_tooltips();
    display_show_hide_toolbar();
+
+   /* update file_conversion data */
+   file_convert_prefs_changed ();
 }
 
 /* save current window size */
@@ -716,7 +876,7 @@ void prefs_window_update_default_sizes (void)
 	gint defx, defy;
 	gtk_window_get_size (GTK_WINDOW (prefs_window), &defx, &defy);
 	prefs_set_int("size_prefs.x", defx);
-  prefs_set_int("size_prefs.y", defy);
+	prefs_set_int("size_prefs.y", defy);
     }
 }
 
@@ -762,7 +922,7 @@ void prefs_window_delete(void)
 
     /* save current window size */
     gtk_window_get_size (GTK_WINDOW (prefs_window), &defx, &defy);
-	  prefs_set_int("size_prefs.x", defx);
+    prefs_set_int("size_prefs.x", defx);
     prefs_set_int("size_prefs.y", defy);
 
     /* close the window */
@@ -776,25 +936,7 @@ void prefs_window_delete(void)
 void
 prefs_window_ok (void)
 {
-    gint defx, defy;
-    GtkWidget *nb;
-
-    /* Commit temp prefs to prefs table */
-    temp_prefs_apply(temp_prefs);
-    temp_lists_apply(temp_lists);
-  
-    /* save current settings */
-    prefs_window_set ();
-
-    /* save current notebook page */
-    nb = gtkpod_xml_get_widget (prefs_window_xml, "notebook");
-    prefs_set_int("last_prefs_page",gtk_notebook_get_current_page (
-		  GTK_NOTEBOOK (nb)));
-
-    /* save current window size */
-    gtk_window_get_size (GTK_WINDOW (prefs_window), &defx, &defy);
-    prefs_set_int("size_prefs.x", defx);
-    prefs_set_int("size_prefs.y", defy);
+    prefs_window_apply ();
 
     /* close the window */
     if(prefs_window)
@@ -836,6 +978,7 @@ prefs_window_apply (void)
 
    ----------------------------------------------------------------- */
 
+
 void
 on_sorting_clicked                     (GtkButton       *button,
 					gpointer         user_data)
@@ -939,25 +1082,34 @@ on_prefs_apply_clicked                 (GtkButton       *button,
 }
 
 
+void on_file_convert_max_threads_num_spinbutton_value_changed (
+    GtkSpinButton *spinbutton,
+    gpointer       user_data)
+{
+    temp_prefs_set_int (temp_prefs, FILE_CONVERT_MAX_THREADS_NUM,
+			gtk_spin_button_get_value_as_int (spinbutton));
+}
+
+void on_file_convert_maxdirsize_spinbutton_value_changed (
+    GtkSpinButton *spinbutton,
+    gpointer       user_data)
+{
+    temp_prefs_set_double (temp_prefs, FILE_CONVERT_MAXDIRSIZE,
+			   gtk_spin_button_get_value (spinbutton));
+}
+
+
 void
-on_cfg_md5tracks_toggled                (GtkToggleButton *togglebutton,
+on_cfg_sha1tracks_toggled                (GtkToggleButton *togglebutton,
 					gpointer         user_data)
 {
     gboolean val = gtk_toggle_button_get_active(togglebutton);
     GtkWidget *w = gtkpod_xml_get_widget (prefs_window_xml, "cfg_show_duplicates");
 
-    temp_prefs_set_int(temp_prefs, "md5", val);
+    temp_prefs_set_int(temp_prefs, "sha1", val);
     if(w)	gtk_widget_set_sensitive (w, val);
 }
 
-void
- on_cfg_block_display_toggled           (GtkToggleButton *togglebutton,
-					gpointer         user_data)
-{
-    temp_prefs_set_int(temp_prefs, "block_display",
-		       gtk_toggle_button_get_active(togglebutton));
-}
-
 void
 on_cfg_update_existing_toggled         (GtkToggleButton *togglebutton,
 					gpointer         user_data)
@@ -1047,6 +1199,45 @@ static void standard_toggle_toggled (GtkToggleButton *togglebutton,
 			gtk_toggle_button_get_active (togglebutton));
 }
 
+static void on_convert_toggle_toggled (GtkToggleButton *togglebutton,
+				     gpointer not_used)
+{
+    GtkTable *parent;
+    GList *list_item;
+    GtkTableChild *child;
+    guint16 row = 0;
+    gboolean active;
+
+    parent = (GtkTable*)gtk_widget_get_parent (GTK_WIDGET (togglebutton));    
+    g_return_if_fail (GTK_IS_TABLE (parent));
+
+    /* Find row of togglebutton */
+    list_item = parent->children;
+    while (list_item) {
+        child = (GtkTableChild*)list_item->data;
+        if (child->widget == (GtkWidget*)togglebutton) {
+            row = child->top_attach;
+            break;
+        }
+        list_item = g_list_next (list_item);
+    }
+
+    active = gtk_toggle_button_get_active (togglebutton);
+
+    temp_prefs_set_int (temp_prefs, gtk_widget_get_name (GTK_WIDGET (togglebutton)), 
+			active);
+
+    /* set sensitivity of all widgets on togglebuttons row, except togglebutton */
+    list_item = parent->children;
+    while (list_item) {
+        child = (GtkTableChild*)list_item->data;
+        if (child->top_attach == row && child->widget != (GtkWidget*)togglebutton) 
+            gtk_widget_set_sensitive (child->widget, active);
+        list_item = g_list_next (list_item);
+    }
+}
+
+
 void
 on_sync_confirm_delete_toggled     (GtkToggleButton *togglebutton,
 				    gpointer         user_data)
@@ -1071,14 +1262,6 @@ on_sync_confirm_delete_toggled     (GtkToggleButton *togglebutton,
 }
 
 
-void
-on_cfg_autoimport_toggled              (GtkToggleButton *togglebutton,
-					gpointer         user_data)
-{
-    temp_prefs_set_int(temp_prefs, "autoimport",
-		       gtk_toggle_button_get_active(togglebutton));
-}
-
 void
 on_charset_combo_entry_changed          (GtkEditable     *editable,
 					gpointer         user_data)
@@ -1234,7 +1417,6 @@ on_cfg_display_tooltips_prefs_toggled  (GtkToggleButton *togglebutton,
 		       gtk_toggle_button_get_active  (togglebutton));
 }
 
-
 void
 on_cfg_display_toolbar_toggled         (GtkToggleButton *togglebutton,
 					gpointer         user_data)
@@ -1378,13 +1560,6 @@ void prefs_window_set_toolbar_style (GtkToolbarStyle style)
 {
 }
 
-void
-on_cfg_temporarily_disable_sorting     (GtkToggleButton *togglebutton,
-                                        gpointer         user_data)
-{
-    temp_prefs_set_int(temp_prefs, "tmp_disable_sort",
-		       gtk_toggle_button_get_active(togglebutton));
-}
 void
 on_cfg_startup_messages                (GtkToggleButton *togglebutton,
                                         gpointer         user_data)
@@ -1429,6 +1604,64 @@ on_mserv_username_entry_changed              (GtkEditable     *editable,
     g_free(val);
 }
 
+/* last.fm callbacks */
+void
+on_lastfm_active_toggled              (GtkToggleButton *togglebutton,
+					gpointer         user_data)
+{
+#if 0
+    int x = gtk_toggle_button_get_active(togglebutton);
+    GtkWidget *u = gtkpod_xml_get_widget  (prefs_window_xml, "lastfm_username");
+    GtkWidget *p = gtkpod_xml_get_widget  (prefs_window_xml, "lastfm_password");
+    temp_prefs_set_int(temp_prefs, "lastfm_active", x);
+
+    gtk_widget_set_sensitive (u, x);
+    gtk_widget_set_sensitive (p, x);
+#endif
+}
+
+void
+on_lastfm_username_entry_changed          (GtkEditable     *editable,
+					gpointer         user_data)
+{
+#if 0
+    gchar *uname;
+    uname = gtk_editable_get_chars (editable, 0, -1);
+    temp_prefs_set_string(temp_prefs, "lastfm_username", uname);
+    g_free (uname);
+#endif
+}
+
+void
+on_lastfm_password_entry_changed          (GtkEditable     *editable,
+					gpointer         user_data)
+{
+#if 0
+    gchar *upass, *upass_old;
+
+    upass = gtk_editable_get_chars (editable, 0, -1);
+    upass_old = prefs_get_string ("lastfm_password");
+
+    if (!upass_old || (strcmp (upass, upass_old) != 0))
+    {
+	unsigned char sig[16];
+	unsigned char md5[64];
+	gint i;
+
+	md5_buffer((const char *)upass, strlen(upass), sig);
+
+	for (i=0; i<16; ++i)
+	{
+	    snprintf (&md5[i*2], 3, "%02x", sig[i]);
+	}
+
+	temp_prefs_set_string(temp_prefs, "lastfm_password", md5);
+    }
+    g_free (upass);
+    g_free (upass_old);
+#endif
+}
+
 
 /* ------------------------------------------------------------ *\
  *                                                              *
@@ -1658,7 +1891,7 @@ void sort_window_update (void)
 	if (w)
 	    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w), TRUE);
 
-	switch (prefs_get_int("tm_sort_"))
+	switch (prefs_get_int("tm_sort"))
 	{
 	case SORT_ASCENDING:
 	    w = gtkpod_xml_get_widget (sort_window_xml, "tm_ascend");
@@ -1775,11 +2008,11 @@ static void sort_window_set ()
 	pm_sort (val);
     if (temp_prefs_get_int_value(sort_temp_prefs, "st_sort", &val))
 	st_sort (val);
-    if (temp_prefs_get_int_value(sort_temp_prefs, "tm_sort_", NULL) ||
+    if (temp_prefs_get_int_value(sort_temp_prefs, "tm_sort", NULL) ||
 	(sortcol_old != sortcol_new))
     {
 	tm_sort_counter (-1);
-	tm_sort (prefs_get_int("tm_sortcol"), prefs_get_int("tm_sort_"));
+	tm_sort (prefs_get_int("tm_sortcol"), prefs_get_int("tm_sort"));
     }
     /* if auto sort was changed to TRUE, store order */
     if (!temp_prefs_get_int(sort_temp_prefs, "tm_autostore"))
@@ -2002,7 +2235,7 @@ void sort_window_set_st_sort (gint val)
 
 void sort_window_set_tm_sort (gint val)
 {
-    temp_prefs_set_int(sort_temp_prefs, "tm_sort_", val);
+    temp_prefs_set_int(sort_temp_prefs, "tm_sort", val);
 }
 
 void sort_window_set_case_sensitive (gboolean val)
